/*
   Multi-Texturing Demo.
   Created by the Programming Ace.
   www.UltimateGameProgramming.com

   The purpose of this demo is to do multitexturing.
   Multitexturing is blending two images together.  First to do multitexturing we
   must load the multitexture functions.  The function extensions are in the
   glext.h header file.

   Next we load two image.  Then we use our multitexture functions to start
   multitexturing and to specify texture coordinates.  Multitexturing can add a
   bit of detail to your scenes so I hope you enjoy.  You will need the glext.h
   header file.  You can get the lattest version of the file from:

   http://oss.sgi.com/projects/ogl-sample/ABI/glext.h
*/


// Demo: Multi-Texturing.


#define WIN32_LEAN_AND_MEAN      // This trims down the libraries.
#define VC_LEANMEAN              // Trims even farther.


#include<windows.h>              // The windows header file.
#include<gl/glu.h>               // Opengl utilities.
#include<gl/glext.h>             // OpenGL extensions.
#include"CTGALoader.h"           // .tga loading class.


// Function Prototypes...
void RenderScene();              // Renders the scene.
bool InitializeGL();             // Initialize the window.
void Shutdown();                 // Release all resources.


// Global device context.
HDC g_HDC;

// This object holds all textures and texture related functions.
CTGALoader Image1;
CTGALoader Image2;

// Multitexture functions.
PFNGLACTIVETEXTUREARBPROC glActiveTextureARB = 0;
PFNGLMULTITEXCOORD2FARBPROC glMultiTexCoord2fARB = 0;


void SetupPixelFormat(HDC hDC)
{
   int nPixelFormat;

   static PIXELFORMATDESCRIPTOR pfd = {
         sizeof(PIXELFORMATDESCRIPTOR),   // size of structure.
         1,                               // always 1.
         PFD_DRAW_TO_WINDOW |             // support window
         PFD_SUPPORT_OPENGL |             // support OpenGl
         PFD_DOUBLEBUFFER,                // support double buffering
         PFD_TYPE_RGBA,                   // support RGBA
         16,                              // bit color mode
         0, 0, 0, 0, 0, 0,                // ignore color bits
         0,                               // no alpha buffer
         0,                               // ignore shift bit
         0,                               // no accumulation buffer
         0, 0, 0, 0,                      // ignore accumulation bits.
         16,                              // number of depth buffer bits.
         0,                               // number of stencil buffer bits.
         0,                               // 0 means no auxiliary buffer
         PFD_MAIN_PLANE,                  // The main drawing plane
         0,                               // this is reserved
         0, 0, 0 };                       // layer masks ignored.

   // this chooses the best pixel format and returns index.
   nPixelFormat = ChoosePixelFormat(hDC, &pfd);

   // This set pixel format to device context.
   SetPixelFormat(hDC, nPixelFormat, &pfd);
}


LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
   static HGLRC hRC;                      // Rendering context.
   static HDC hDC;                        // Device context.
   int width, height;                     // The window width and height.

   switch(message)
      {
         case WM_CREATE:                  // Windows creation.
            hDC = GetDC(hwnd);            // This gets the device context for our window.
            g_HDC = hDC;                  // Assigns the global device context to this one.
            SetupPixelFormat(hDC);        // Call the pixel format function.

            hRC = wglCreateContext(hDC);  // Creates the rendering context.
            wglMakeCurrent(hDC, hRC);     // Makes the rendering context.
            return 0;
            break;

         case WM_CLOSE:                   // Close message.
         case WM_DESTROY:
            wglMakeCurrent(hDC, NULL);
            wglDeleteContext(hRC);        // Deletes the rendering context.

            PostQuitMessage(0);           // Says close the program.
            return 0;
            break;

         case WM_SIZE:                    // re-size message.
            height = HIWORD(lParam);      // This gets the height of the window.
            width = LOWORD(lParam);       // This gets the width of the window.

            if(height==0)                 // we don't want it to be possible for a
               {                          // height of 0.  If it is 0 me make it 1.
                  height = 1;
               }

            glViewport(0, 0, width, height);// resets the viewport to new dimensions.
            glMatrixMode(GL_PROJECTION);    // Sets the projection matrix.
            glLoadIdentity();               // Reset the modelview matrix.

            // Calculate the aspect ratio of the window.
            gluPerspective(45.0f, (GLfloat)width/(GLfloat)height, 0.1f, 1000.0f);

            glMatrixMode(GL_MODELVIEW);     // Sets the projection matrix.
            glLoadIdentity();               // Reset the modelview matrix.

            return 0;
            break;

         case WM_KEYDOWN:
		      switch(wParam)
               { 
		            case VK_ESCAPE: // Close the app if the user presses escape.
				         PostQuitMessage(0);
				         break;
               }
            break;

         default:
            break;
      }
   // What this does is pass all of the unhandled messages to DefWindowProc.
   return (DefWindowProc(hwnd, message, wParam, lParam));
}


int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nShowCmd)
{
   MSG msg;                // A message variable.
   WNDCLASSEX windowClass; // Your Window class.
   HWND hwnd;              // The Window handle.
   bool isFinished;        // This will be used to check if the program is done or not.

   // This is the Window class.
   windowClass.cbSize = sizeof(WNDCLASSEX);        // size of the WNDCLASSEX structure.
   windowClass.style = CS_HREDRAW | CS_VREDRAW;    // style of the window.
   windowClass.lpfnWndProc = WndProc;              // Address to the windows procedure.
   windowClass.cbClsExtra = 0;                     // Extra class information.
   windowClass.cbWndExtra = 0;                     // Extra window information.
   windowClass.hInstance = hInstance;              // Handle of application Instance.
   // Handle of application Icon.
   windowClass.hIcon = LoadIcon(NULL, IDI_APPLICATION);
   windowClass.hCursor = LoadCursor(NULL, IDC_ARROW);// mouse cursor
   windowClass.hbrBackground = NULL;               // background color.
   windowClass.lpszMenuName = NULL;                // name of the main menu.
   windowClass.lpszClassName = "UltimateGameProgrammingClass";// window class name.
   // icon when minimized.
   windowClass.hIconSm = LoadIcon(NULL, IDI_APPLICATION);

   // You must register you class with Windows.
   if(!RegisterClassEx(&windowClass)) return 0;

   // Create the window.
   hwnd = CreateWindowEx(NULL,// The extended window style.
                        "UltimateGameProgrammingClass",// window Class name.
                        "Multi-Texturing Demo - By the Programming Ace.",// window name.
                        WS_OVERLAPPEDWINDOW | WS_VISIBLE |// The window style.
                        WS_SYSMENU | WS_CLIPCHILDREN |// window style.
                        WS_CLIPSIBLINGS,// window style.
                        100, 100,// window x, y coordinate.
                        640, 480,// window width and height.
                        NULL,// handle to parent window.
                        NULL,// menu.
                        hInstance,// handle to app instance.
                        NULL);   // pointer to window creation data.
   
 
   // If there was an error with creating the window, then close the program.
   if(!hwnd) return 0;

   ShowWindow(hwnd, SW_SHOW); // This shows the window.
   UpdateWindow(hwnd);        // This forces a paint message.

   isFinished = false;        // False = running, True = not running.

   // If initialize fail (return false), then we don't want the program to run.
   if(!InitializeGL()) isFinished = true;

   // This is the messsage loop.
   while(!isFinished)
      {
         if(PeekMessage(&msg, NULL, NULL, NULL, PM_REMOVE))
            {
               // If a quit message is received then stop rendering and quit the app.
               if(msg.message == WM_QUIT)
                  {
                     isFinished = true;
                  }

               TranslateMessage(&msg);  // Translate any messages.
               DispatchMessage(&msg);   // Dispatch any messages.
            }
         else
            {
               RenderScene();          // Render a frame.
            }
      }

   Shutdown();
   return (int)msg.wParam;
}


bool InitializeGL()
{
   // Initialize OpenGL states.
   glClearColor(0.0, 0.0, 0.0, 0.0);

   glShadeModel(GL_SMOOTH);
	glEnable(GL_TEXTURE_2D);

   // Load our images.
   if(!Image1.LoadTGA("image.tga")) return false;

	glGenTextures(1, &Image1.ID);
   glBindTexture(GL_TEXTURE_2D, Image1.ID);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGB, Image1.imageWidth, Image1.imageHeight,
	                  GL_RGB, GL_UNSIGNED_BYTE, Image1.image);

   if(!Image2.LoadTGA("image2.tga")) return false;
	glGenTextures(1, &Image2.ID);
   glBindTexture(GL_TEXTURE_2D, Image2.ID);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGB, Image2.imageWidth,
	                  Image2.imageHeight, GL_RGB, GL_UNSIGNED_BYTE,
	                  Image2.image);


   // With this we will load the list of extensions the hardware supports.
   char *extension = (char*)glGetString(GL_EXTENSIONS);

   // Check for multitexture support.
   if(strstr(extension, "GL_ARB_multitexture") == 0) return false;

   // Load the multitexture funtion and test if all went well.
   glActiveTextureARB = (PFNGLACTIVETEXTUREARBPROC)wglGetProcAddress("glActiveTextureARB");
	glMultiTexCoord2fARB	= (PFNGLMULTITEXCOORD2FARBPROC)wglGetProcAddress("glMultiTexCoord2fARB");

   // Error checking to make sure functions loaded.
   if(!glActiveTextureARB || !glMultiTexCoord2fARB) return false;

   return true;
}


void RenderScene()
{
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);   // Clears the screen.
   glLoadIdentity();                   //Reset modelview matrix for new frame.

   // To specify we want to do multitexturing we first must activate at least
   // two texture units.  This is done by calling glActiveTextureARB like so...

   // Set the first texture unit for the first image..
	glActiveTextureARB(GL_TEXTURE0_ARB);
	glEnable(GL_TEXTURE_2D);
   glBindTexture(GL_TEXTURE_2D, Image1.ID);

	// Set the second texture unit for the second image..
	glActiveTextureARB(GL_TEXTURE1_ARB);
	glEnable(GL_TEXTURE_2D);
   glBindTexture(GL_TEXTURE_2D, Image2.ID);


   // Draw the polygon with the two images combined.
   glBegin(GL_QUADS);

      // The next two lines sets the texture coordinates for the two images.
      // As you can see you must set the texture coordinates for each vertex just
      // like with normal texture mapping.  This time you must set two pairs for each image.
      glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f, 1.0f);
      glMultiTexCoord2fARB(GL_TEXTURE1_ARB, 1.0f, 1.0f);
      glVertex3f(0.5f, 0.5f, -2.0f);

      glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f, 1.0f);
      glMultiTexCoord2fARB(GL_TEXTURE1_ARB, 0.0f, 1.0f);
      glVertex3f(-0.5f, 0.5f, -2.0f);

      glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f, 0.0f);
      glMultiTexCoord2fARB(GL_TEXTURE1_ARB, 0.0f, 0.0f);
      glVertex3f(-0.5f, -0.5f, -2.0f);

      glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f, 0.0f);
      glMultiTexCoord2fARB(GL_TEXTURE1_ARB, 1.0f, 0.0f);
      glVertex3f(0.5f, -0.5f, -2.0f);

   glEnd();


   // Disable all texture units.
   glActiveTextureARB(GL_TEXTURE1_ARB);
	glBindTexture(GL_TEXTURE_2D, NULL);

	glActiveTextureARB(GL_TEXTURE0_ARB);
	glBindTexture(GL_TEXTURE_2D, NULL);


   SwapBuffers(g_HDC);
}


void Shutdown()
{
   Image1.FreeImage();
   Image2.FreeImage();

   glDeleteTextures(1, &Image1.ID);
   glDeleteTextures(1, &Image2.ID);
}


// Recap:

// Now you know how to multi-texture.  If your hardware does not support it
// then you will not see anything.  For those of you that do have the
// right hardware as you can see that the two images blended nicely.  You can
// play around with different images to see how it looks.  Remember to do multitexturing
// first we load 2 or more image.  Next we load the multi-texture OpenGL functions.
// Then we use those function just like any other OpenGL function to specify the
// texture coordinates for the textures.  Once specified and we activated both texture
// units (GL_TEXTURE0_ARB and GL_TEXTURE1_ARB) then you are all set.


// Copyright June 2004
// All Rights Reserved!
// Allen Sherrod
// ProgrammingAce@UltimateGameProgramming.com
// www.UltimateGameProgramming.com
